ModeIconColors = {
	Normal		= color("#FFEE00"), -- yellow
	Rave		= color("#db93ff"), -- violet
	Nonstop		= color("#5ca9ff"), -- blue
	Oni			= color("#00f1e2"), -- cyan
	Endless		= color("#b4c3d2"), -- steel
}
